@@ -12,7 +12,7 @@ use strict;
 use warnings;
 use Carp qw(croak);
 
-our $VERSION = '0.02';
+our $VERSION = '0.04';
 
 =pod
 
@@ -24,7 +24,7 @@ Parse::Lnk - A cross-platform, depencency free, Windows shortcut (.lnk) meta dat
 
 =head1 VERSION
 
-Version 0.02
+Version 0.04
 
 =cut
 
@@ -104,45 +104,93 @@ our $map = { # Tag names made up based on the docs
 
 sub resolve_lnk {
     my $filename = shift;
-    croak "$filename is not a valid filename" unless defined($filename) and -f $filename;
-    my $data = __PACKAGE__->from($filename);
-    if ($data) {
-        return $data->{base_path};
-    }
-    return;
+    my $l = __PACKAGE__->new (
+        filename => $filename,
+        resolve  => 1,
+    );
+    $l->_parse;
+    $l->{base_path};
 }
 
 sub parse_lnk {
     my $filename = shift;
-    croak "$filename is not a valid filename" unless defined($filename) and -f $filename;
-    __PACKAGE__->from($filename);
+    my $l = __PACKAGE__->new (
+        filename => $filename,
+    );
+    $l->_parse;
+    return if $l->{error};
+    $l;
 }
 
 sub new {
     my $class = shift;
+    if (@_ and @_ % 2) {
+        croak "This method expects (name => value) arguments. Odd number of arguments received";
+    }
     my $self = {
-        error => undef,
+        @_,
     };
     bless $self, $class;
 }
 
 sub from {
-    my $self = shift()->new;
+    my $self = shift;
     my $filename = shift;
-    my @caller = caller;
-    croak "'parse' is an instance method, can't invoke it through a plain package name" unless ref($self) eq __PACKAGE__;
-    unless ($filename and -f $filename) {
-        $self->{error} = "Not a file: $filename";
+    $self = $self->new (
+        filename => $filename,
+    ) unless ref $self;
+    $self->_parse;
+    return if $self->{error};
+    $self;
+}
+
+sub parse {
+    my $self = shift;
+    my $filename = shift;
+    $self = $self->new (
+        filename => $filename,
+    ) unless ref $self;
+    $self->_parse;
+    return if $self->{error};
+    $self;
+}
+
+sub _reset {
+    my $self = shift;
+    return unless ref $self;
+    for my $k (keys %$self) {
+        delete $self->{$k};
+    }
+    $self;
+}
+
+sub _parse {
+    my $self = shift;
+    my $filename = $self->{filename};
+    my $resolve = $self->{resolve};
+    $self->_reset;
+    if (not defined $filename) {
+        $self->{error} = 'A filename is required';
         return;
     }
-    open my $in, '<', $filename or croak "Can't open file $filename for reading\n";
-    binmode $in;
-    $self->{_fh} = $in;
+    if (not -f $filename) {
+        $self->{error} = "Not a file";
+        return;
+    }
+    if (open my $in, '<', $filename) {
+        binmode $in;
+        $self->{_fh} = $in;
+    } else {
+        # We set error before croak, in case this call is being eval'ed
+        $self->{error} = "Can't open file '$filename' for reading";
+        croak $self->{error};
+    }
     
     my $header = $self->_read_unpack(0, 1);
     if ($header ne '4c') {
         $self->{error} = 'Invalid Lnk file header';
-        close $in;
+        close $self->{_fh};
+        delete $self->{_fh};
         return;
     }
     
@@ -311,6 +359,11 @@ sub from {
         # Read base path data upto NULL term 
         my $bp_data = $self->_read_null_term($base_path_off);
         $self->{base_path} = $bp_data;
+        if ($resolve) {
+            close $self->{_fh};
+            delete $self->{_fh};
+            return $self;
+        }
     }
 
     # Network Volume Table
@@ -326,11 +379,18 @@ sub from {
         if ($net_share_name_loc ne "20") {
             close delete $self->{_fh};
             $self->{error} = 'Error: NSN ofset should always be 14h';
-            return my $h = {%$self};
+            close $self->{_fh};
+            delete $self->{_fh};
+            return $self;
         }
         $net_share_name_loc = $net_vol_off + $net_share_name_loc;
         my $net_share_name = $self->_read_null_term($net_share_name_loc);
         $self->{base_path} = $net_share_name;
+        if ($resolve) {
+            close $self->{_fh};
+            delete $self->{_fh};
+            return $self;
+        }
 
         # Mapped Network Drive Info
         my $net_share_mdrive = $net_vol_off + 12;
@@ -367,11 +427,18 @@ sub from {
         if ($net_share_name_loc ne "20") {
             close delete $self->{_fh};
             $self->{error} = 'Error: NSN ofset should always be 14h';
-            return my $h = {%$self};
+            close $self->{_fh};
+            delete $self->{_fh};
+            return $self;
         }
         $net_share_name_loc = $net_vol_off + $net_share_name_loc;
         my $net_share_name = $self->_read_null_term($net_share_name_loc);
         $self->{base_path} = $net_share_name;
+        if ($resolve) {
+            close $self->{_fh};
+            delete $self->{_fh};
+            return $self;
+        }
 
         # Mapped Network Drive Info
         my $net_share_mdrive = $net_vol_off + 12;
@@ -427,7 +494,7 @@ sub from {
         $self->{icon_filename} = $addnl_text;
     }
     close delete $self->{_fh};
-    my $h = {%$self};
+    $self;
 }
 
 sub _add_info {
@@ -485,6 +552,7 @@ sub _MStime_to_unix {
     $mstime_dec *= .0000001;
     # Add difference in epochs
     $mstime_dec -= $MSConversion;
+    sprintf '%0.3f', $mstime_dec;
 }
 
 sub _reverse_hex {
@@ -546,42 +614,157 @@ Its use is very simple:
     
     # $data is now a hashref if the file was parsed successfully.
     # undef if not.
-
-=head1 EXPORT
-
-Nothing is exported by default. But you can import more convenient functions:
-
+    
+    ##########
+    # Or ... #
+    ##########
+    
     use Parse::Lnk qw(parse_lnk);
     
     my $data = parse_lnk $filename;
     
     # $data is now a hashref if the file was parsed successfully.
     # undef if not.
-
-Or:
-
+    
+    ##########
+    # Or ... #
+    ##########
+    
     use Parse::Lnk qw(resolve_lnk);
     
     my $path = resolve_lnk $filename;
     
     # $path is now a string with the path the lnk file points to.
     # undef if the lnk file was not parsed successfully.
+    
+    ###############################################################
+    # Or, if you want a little more information/control on errors #
+    ###############################################################
+    
+    use Parse::Lnk;
+    
+    my $lnk = Parse::Lnk->new;
+    
+    $lnk->parse($filename) or die $lnk->{error};
+    
+    # Or:
+    
+    $lnk->parse($filename);
+    
+    if ($lnk->{error}) {
+        # ... do your own error handling;
+    }
+    
 
 
 
-=head1 SUBROUTINES/METHODS
+=head1 EXPORT
 
-=head2 from
+Nothing is exported by default. You can explicitly import this functions:
 
-The only available method is C<from>. You can use it if you do not import
-the convinience functions previously described:
+=head2 parse_lnk($filename)
+
+This will return a Parse::Lnk instance, which is a hashref. The keys in that
+hashref depend on the data that was parsed from the .lnk file.
+
+It will return C<undef> on error.
+
+    use Parse::Lnk qw(parse_lnk);
+    
+    my $lnk = parse_lnk $filename;
+    
+    if ($lnk) {
+        print "$filename points to path $lnk->{base_path}\n";
+        
+        my $create_date = localtime $lnk->{create_time};
+        print "$filename was created on $create_date";
+    } else {
+        print "Could not parse $filename";
+    }
+
+=head2 resolve_lnk($filename)
+
+This will return the path the .lnk file is pointing to.
+
+It will return C<undef> on error.
+
+    use Parse::Lnk qw(resolve_lnk);
+    
+    my $path = resolve_lnk $filename;
+    
+    if ($path) {
+        print "$filename points to path $path";
+    } else {
+        print "Could not parse $filename";
+    }
+
+
+=head1 METHODS
+
+You can create a C<Parse::Lnk> instance and call a few methods on it. This
+may give you more control/information when something goes wrong while parsing
+the file.
+
+=head2 new
+
+This creates a new instance. You can pass the C<filename> value as argument,
+or you can set/change it later.
 
     use Parse::Lnk;
     
-    my $data = Parse::Lnk->from($filename);
+    my $lnk = Parse::Lnk->new(filename => $filename);
     
-    # $data is now a hashref if the file was parsed successfully.
-    # undef if not.
+    # or
+    
+    my $lnk = Parse::Lnk->new;
+    $lnk->{filename} = $filename;
+
+=head2 parse
+
+This method will parse the current C<filename> in the instance. You can change
+the value of C<filename> and parse again at any point.
+
+    use Parse::Lnk;
+    
+    my $lnk = Parse::Lnk->new(filename => $filename);
+    
+    $lnk->parse;
+    
+    if ($lnk->{error}) {
+        # handle the error
+    } else {
+        print "$filename points to $lnk->{base_path}";
+    }
+    
+    for my $other_filename (@filenames) {
+        $lnk->{filename} = $other_filename;
+        $lnk->parse;
+        
+        if ($lnk->{error}) {
+            # handle the error
+            next;
+        }
+        
+        print "$other_filename points to $lnk->{base_path}";
+    }
+
+=head2 from
+
+It will return a C<Parse::Lnk> instance, or undef on error. This method was
+written with plain package name calling in mind:
+
+    use Parse::Lnk;
+    
+    my $lnk = Parse::Lnk->from($filename);
+    
+    if ($lnk) {
+        print "$filename points to path $lnk->{base_path}\n";
+        
+        my $create_date = localtime $lnk->{create_time};
+        print "$filename was created on $create_date";
+    } else {
+        print "Could not parse $filename";
+    }
 
 
 =head1 AUTHOR
